/* recstat computes a and b values, confidence intervals
for both data and prediction, and return periods,
incremental and cumulative recurrence relations.
Methodology is outlined in Bollinger et al.(1993), Weichert(1980),
and Berril and Davis (1980).
Usage:  recstat InputTable
Format input table as follows:

	Title
	1234567 sq km
	mag1  mag2  mon1/year1  mon2/year2  #events 
	mag2  mag3  mon2/year2  mon3/year3  #events 
	.
	.
where beginning is day 1 of month1 of year1,
end is last day of month2 of year2.  For example,
beginning of 1990 through end of 1992 is  1/1990 12/1992.
1234567 is the area of the region, in square km.
Smallest magnitude should be mag1.
Zero events in an interval is permitted.
The magnitude intervals should be equal; user is responsible
for this. dm = magnitude interval/2.
Writes data points and their confidence bounds to pts.inc,
pts.cum, and curve.cum. For cumulative results pts.cum and
curve.cum are needed.
Set confidence level and maximum magnitude below.

R. LaForge 3/96.  Version 1.2
*/

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <ctype.h>

#define MAXINTS 15
#define CONFLEVEL 	.95
#define MMAX 		7.5

extern double critchi(double,int);
extern double pochisq(double,int);
extern double poz(double);
extern double critz(double);

main(argc,argv)
	int argc;
	char **argv;
	{
char date1str[8],date2str[8],yrstr[5],infile[20];
char line[80],title[70];
float mag1[MAXINTS],mag2[MAXINTS];
float t[MAXINTS],m[MAXINTS],fprimebeta,fbeta=1.;
float lowinc[MAXINTS],highinc[MAXINTS];
float lowcum[MAXINTS],highcum[MAXINTS];
float cumrate[MAXINTS],floatk,b,area,logarea;
float ainc,acumcalc,acumrbh,acumna;
float mbar=0.,dm,M=0.,beta,betanew=2.0;
float bfact=.4343294,dmag=.05,mag;
float term1,term2,term3,term4,xhigh,xlow;
float term1c,term2c,term3c,term4c;
float alphainc,sdalphainc,cor,D;
float varabm,stdabm,mu,zlow,zhigh;
float vara,varb,sdbinc,sdbcum,covab,covabinc,covabcum;
float stdevb,stdeva,sdainc,sdacum,chisdbinc,chisdbcum;
float beta84,Nainc,Nacum,sdnainc,sdnacum,nm,T,p1,p2;
float alphacum,tcum1,cumpred[MAXINTS],rb1,rb2;
float date1[MAXINTS],date2[MAXINTS],mbarc=0.,sdalphacum;
int k[MAXINTS],kcum[MAXINTS];
int n=0,i,j,K=0,iter=0,df1,df2,Kcum=0,mon,year1,year2;
float cudays[] = {0,31,59,90,120,151,181,212,243,273,304,334,365};
float culdays[] = {0,31,60,91,121,152,182,213,244,274,305,335,366};

FILE *tablefile,*ptsinc,*ptscum,*curvecum;
ptsinc=fopen("pts.inc","w");
ptscum=fopen("pts.cum","w");
curvecum=fopen("curve.cum","w");

/* get input file name */

	if(argc < 2) {
		puts("USAGE:  recstat InputTable");
		exit(0);
		}
	while (--argc)strcpy(infile,*++argv);
	tablefile=fopen(infile,"r");

p1=.5*(1-CONFLEVEL); /* low  */
p2=.5*(1+CONFLEVEL); /* high */
printf("\n         Recstat Results  (Version 1.2)\n");
printf("\n         Copyright R. LaForge (1996)\n\n");

	fgets(line,sizeof line,tablefile);
	sprintf(title,"%s",line);      
	title[strlen(title)] = '\0';
	fscanf(tablefile,"%f%*s%*s",&area);
	logarea=log10(area);

printf("%s",title);
printf("%4.0f sq km\n\n",area);
printf("  Occurrence Table, from file  %s\n\n",infile);

	while(fscanf(tablefile,"%f%f%s%s%d",\
	&mag1[n],&mag2[n],date1str,date2str,&k[n]) != EOF){
		printf("  %4.2f %4.2f %s %s %d\n",\
		mag1[n],mag2[n],date1str,date2str,k[n]);

/* compute start and end dates */

		for(j=0;j<3;j++){
		if(!isdigit(date1str[j])){
		date1str[j]='\0';
		break;
		}}
		mon=atoi(date1str);
		strcpy(yrstr,&date1str[j+1]);
		year1=atoi(yrstr);
	if((year1 % 4 == 0 && year1 % 100 != 0) || year1 % 400 == 0)
		date1[n]=year1+culdays[mon-1]/culdays[12];
		else date1[n]=year1+cudays[mon-1]/cudays[12];

		for(j=0;j<3;j++){
		if(!isdigit(date2str[j])){
		date2str[j]='\0';
		break;
		}}
		mon=atoi(date2str);
		strcpy(yrstr,&date2str[j+1]);
		year2=atoi(yrstr);
	if((year2 % 4 == 0 && year2 % 100 != 0) || year2 % 400 == 0)
		date2[n]=year2+culdays[mon]/culdays[12];
		else date2[n]=year2+cudays[mon]/cudays[12];
	n++;
	}
printf("\n");

/* form cumulative numbers */

	for(i=0;i<n;i++){
		cumrate[i]=0.;
		kcum[i]=0;
		t[i]=date2[i]-date1[i];
		}
	for(i=0;i<n;i++){
	for(j=i;j<n;j++){
		floatk=k[j];
		cumrate[i] += floatk/t[j];
		kcum[i] += k[j];
		}}
	for(i=0;i<n;i++)Kcum += kcum[i];

/* solve for b (same for both incremental and cumulative) */

	for(i=0;i<n;i++){
		dm=(mag2[i]-mag1[i])*.5;
		m[i]=mag1[i]+dm;
		K += k[i];
		M += m[i];
		}
	for(i=0;i<n;i++)mbar += k[i]*m[i]/K;

	printf("Number of earthquakes in sample = %d\n",K);
	printf("Number of cumulative earthquakes in sample = %d\n\n",Kcum);

	/* find beta (Newton's method) */

	while(fabs(fbeta) > .0001){
	beta=betanew;
	term1=0.;term2=0.;term3=0.;term4=0.;
    for(i=0;i<n;i++){
    	term1 += t[i]*exp(-beta*m[i]);
    	term2 += t[i]*m[i]*m[i]*exp(-beta*m[i]);
    	term3 += t[i]*m[i]*exp(-beta*m[i]);
    	term4 += exp(-beta*m[i]);
		}
	fprimebeta=(term3*term3-term1*term2)/(term1*term1);
	fbeta=(term3/term1)-mbar;
	betanew=beta-(fbeta/fprimebeta);
	iter++;
	if(iter > 30){
		puts("beta calculation did not converge");
		exit(0);
		}
	}
	b=beta*bfact;

	/* find Na(inc) and its s.d. */

		Nainc=K*term4/term1;
		sdnainc=sqrt(Nainc/K);

	/* find alphainc and sd (from Bollinger et al. (1993) */

	alphainc=log(K/term1);
	ainc=alphainc*bfact;
	D=term2/term1; 
	sdalphainc=sqrt(D/(K*(D-mbar*mbar)));
	sdainc=sdalphainc*bfact;

	acumna=b*mag1[0]+log10(Nainc);

/* calculate cumulative a from RBH formula */

	rb1=pow(10,b*dm)-pow(10,-b*dm);
	rb2=pow(10,alphainc*bfact)/rb1;
	acumrbh=log10(rb2);

/* calculate cumulative a from scratch */

	tcum1=0.;
	for(i=0;i<n;i++)tcum1 += t[i]*exp(-beta*mag1[i]);
	alphacum=log(Kcum/tcum1);
	acumcalc=alphacum*bfact;

/*	Compute incremental data variance (from Weichert,1980, eqn.11) */

	for(i=0;i<n;i++){
		if(k[i] == 0){
			df2=2;
			lowinc[i]=.5*critchi(p2,df2)/t[i];
		}else{
		df1=2*k[i];
		lowinc[i]= .5*critchi(p1,df1)/t[i];
		df2=2*k[i]+2;
		highinc[i]= .5*critchi(p2,df2)/t[i];
		}}

/* write out cumulative data points and confidence bounds */

	/* do cumulative Gaussian statistics */

	term1c=0.;term2c=0.;term3c=0.;term4c=0.;
    	for(i=0;i<n;i++)mbarc += kcum[i]*mag1[i]/Kcum;
    	for(i=0;i<n;i++){
    	term1c += t[i]*exp(-beta*mag1[i]);
    	term2c += t[i]*mag1[i]*mag1[i]*exp(-beta*mag1[i]);
    	term3c += t[i]*mag1[i]*exp(-beta*mag1[i]);
    	term4c += exp(-beta*mag1[i]);
		}
	 
	D=term2c/term1c;
	vara=D/(Kcum*(D-mbarc*mbarc));
	varb=1/(Kcum*(D-mbarc*mbarc));
	stdeva=sqrt(vara);
	sdacum=stdeva*bfact;
	stdevb=sqrt(varb);
	sdbcum=stdevb*bfact;
	covab=mbarc/(K*(D-mbarc*mbarc));
	covabcum=covab*bfact;
	beta84=(2*Kcum*beta)/critchi(.84,2*Kcum);
	chisdbcum=(beta84-beta)*bfact;

	Nacum=K*term4c/term1c;
	sdnacum=sqrt(Nacum/Kcum);

/*	Compute cumulative data variance (from Weichert,1980, eqn.11) */

	for(i=0;i<n;i++){
		if(kcum[i] == 0){
			df2=2;
			highcum[i]=.5*critchi(p2,df2);
			lowcum[i]=1e-6;
			cumpred[i]=exp(acumna/bfact-beta*mag1[i]);
			cumrate[i]=1e-6;
		}else{
		df1=2*kcum[i];
		df2=2*kcum[i]+2;
		cor=cumrate[i]/kcum[i];
		highcum[i]= .5*critchi(p1,df1)*cor;
		lowcum[i]= .5*critchi(p2,df2)*cor;
		cumpred[i]=exp(acumna/bfact-beta*mag1[i]);
		}}

/* calculate cumulative uncertainties */

	fprintf(ptscum,"Mag>=  data   datalow  datahigh  predlow  predhigh\n");

    	zlow=critz(p1);
    	zhigh=critz(p2);
    for(i=0;i<n;i++){
        varabm=(D-2*mag1[i]*mbarc+mag1[i]*mag1[i])/\
		(Kcum*(D-mbarc*mbarc));
        stdabm=sqrt(varabm);
        xhigh=zhigh*stdabm+log(cumpred[i]);
        xlow=zlow*stdabm+log(cumpred[i]);
		fprintf(ptscum,"%4.2f %e %e %e %e %e\n",\
          mag1[i],cumrate[i],lowcum[i],highcum[i],\
			exp(xlow),exp(xhigh));
        }

/* generate and write predicted cumulative curve
   (Berril and Davis, 1980 formula) */

	fprintf(curvecum,"mag>=  y\n");
	mag=mag1[0];
	term3=exp(-beta*MMAX);
	term4=1/(1-term3);
	T=pow(10,acumna);
	while(mag<MMAX){
		nm=term4*T*(exp(-beta*mag)-term3);
		fprintf(curvecum,"%4.2f  %10.4e\n",mag,nm);
		mag += dmag;
		}

    /* do incremental Gaussian statistics (from Bollinger et al., 1993) */

	D=term2/term1;
	vara=D/(K*(D-mbar*mbar));
	varb=1/(K*(D-mbar*mbar));
	stdeva=sqrt(vara);
	sdainc=stdeva*bfact;
	stdevb=sqrt(varb);
	sdbinc=stdevb*bfact;
	covab=mbar/(K*(D-mbar*mbar));
	covabinc=covab*bfact;
	beta84=(2*K*beta)/critchi(.84,2*K);
	chisdbinc=(beta84-beta)*bfact;

/* print output */

printf("incremental a =%6.3f  (= %6.3f normalized to sq km/yr)\n",\
	ainc,ainc-logarea);
printf("cumulative a = %6.3f\  (= %6.3f normalized to sq km/yr)\n",\
	acumna,acumna-logarea);
/*printf("cumulative a = %6.3f (Herrmann formula)\n",acumrbh);
printf("cumulative a = %6.3f (calculated)\n",acumcalc); */
printf("\nb = %5.3f\n\n",b);

puts("Parameter Uncertainties\n\n  Incremental Recurrence");
printf("    Standard Deviation of a = %5.3f\n",sdainc);
printf("    Standard Deviation of b (Gaussian) = %5.3f\n",sdbinc);
printf("    chi-square 84th minus mean (b) = %5.3f\n",chisdbinc);
printf("    Covariance of a and b = %5.3f\n",covabinc);
puts("  Cumulative Recurrence");
printf("    Standard Deviation of a = %5.3f\n",sdacum);
printf("    Standard Deviation of b (Gaussian) = %5.3f\n",sdbcum);
printf("    chi-square 84th minus mean (b) = %5.3f\n",chisdbcum);
printf("    Covariance of a and b = %5.3f\n\n",covabcum);

	printf("Observed rate for magnitude >= %4.2f = %6.3f\n",\
		mag1[0],cumrate[0]);
	printf("Maximum Likelihood rate for magnitude >= %4.2f = %6.3f\n",\
		mag1[0],Nainc);
	printf("Standard Deviation of above rate: %5.3f (incremental)\n",sdnainc); 
	printf("                                  %5.3f (cumulative)\n\n",sdnacum); 

/*	printf("\nNa(inc)= %6.3f ; standard deviation of Na(inc)= %5.3f \n",\
		Nainc,sdnainc);
	printf("Na(cum)= %6.3f ; standard deviation of Na(cum)= %5.3f \n\n",\
		Nacum,sdnacum); */

    printf("Return Periods and %4.1f %% Confidence Bounds\n\n",\
		CONFLEVEL*100);
    printf("  Mag\t\tPredicted  High    Low  Observed\n\n");
	fprintf(ptsinc,"Magint  data      pred   datalow     datahigh\
    predlow   predhigh\n");

    for(i=0;i<n;i++){
        varabm=(D-2*m[i]*mbar+m[i]*m[i])/(K*(D-mbar*mbar));
        stdabm=sqrt(varabm);
        mu=alphainc-beta*m[i];
        xhigh=zhigh*stdabm+mu;
        xlow=zlow*stdabm+mu;
		if(k[i] == 0.){
        printf("%4.2f-%4.2f\t%7.2f\t%7.2f\t%7.2f\t%7.2f\n",\
            m[i]-dm,m[i]+dm,1/exp(mu),1/exp(xlow),1/exp(xhigh),0.);
		fprintf(ptsinc,"%4.2f %e %e %e %e %e %e\n",\
          m[i],1e-6,exp(mu),1e-6,lowinc[i],exp(xlow),exp(xhigh));
		}
        else{ printf("%4.2f-%4.2f\t%7.2f\t%7.2f\t%7.2f\t%7.2f\n",\
            m[i]-dm,m[i]+dm,1/exp(mu),1/exp(xlow),1/exp(xhigh),\
            t[i]/k[i]);
		fprintf(ptsinc,"%4.2f %e %e %e %e %e %e\n",\
		m[i],k[i]/t[i],exp(mu),highinc[i],lowinc[i],exp(xlow),exp(xhigh)); 
		}
        }
		printf("\n");

}
