c                       SEISRISK IIIx
c             Modified version by R. LaForge 3/94
c
c      computer program for determining ground motions
c          for use in seismic hazard mapping
c
c      ******Allows locations of future earthquakes in seismic
c         source zones to have a normal distribution*******
c
c      SEISRISK III is a revision of SEISRISK II, documented in Open
c         File Report 82-293.
c
c       See model description of SEISRISK III and additional
c          documentation in USGS Bulletin 1772
c
c      files:
c         standard input: input file
c         file 16:  outputs: filename= riskx.out
c         file 17:  risk.table file for use with relcon (added by RL)
c         file 02:  summary for plots--see text
c         file 03:  accelerations saved on file 03 for use in next
c                   run if next run is a continuation
c
c                           inputs
c
c  1. title-up to 80 characters   (free field)
c  2. isw, sigmax   (free field)
c       isw=0 new run (usual case)  isw=1 restart or continue
c             from previous run
c       sigmax=maximum standard deviation (km) for earthquake
c             location variability in seismic source zones for
c             this run.
c  3. prob,ntims, jtim(1),jtim(2)....jtim(ntims)
c    acceleration is sought for which there is probability prob of
c    not being exceeded in jtim(1), jtim(2), ...jtim(ntim)
c    years
c    prob = extreme probability in decimal
c    ntims = number of times for which calculation is done
c    jtim(i)  =  durations  (years) for which extreme motions are to be
c            calculated at the prob probability level, 1 <= i <= ntims
c  4. scale, dsw, sd, inos  (free field)
c        scale=scaling factor for ground motion boxes
c        scale=1 for motions .02 to 1.0--scale accordingly
c        dsw:  =1 if inputs are degrees and minutes
c              =0 if inputs are decimal degrees
c        sd=standard deviation in log acceleration
c             for acceleration variability around mean value
c        inos=1: divide magnitude interval in half and do calculations
c             at twice as many magnitudes (Assumes Gutenberg-Richter
c             b-value is same for all intervals.)
c        inos= any number other than 1: do calculations for original
c             magnitude intervals
c  5. x1,y1, x2,y2 (long,lat) in decimal degrees (free field)
c     transform great circle thru (x1,y1), (x2,y2), (0,0)
c     to equator for new coordinate system
c  6. fl1,ph1,fl2,ph2,phinc
c     (fl1,ph1) upper left    (fl2,ph2) lower right  (long, lat)
c     corners of seismic felt region for risk computation
c     rectangular region with sides parallel to arc
c     of great circle thru (x1,y1), (x2,y2) (0,0) --defined by 5.
c     other two sides perpendicular and thru given end points
c
c         + (fl1,ph1)
c        1     .
c     (x1,y1) 1     .
c    .             1     .
c        .              1     .
c            .               1     .
c                .       .        1
c           (fl2,ph2)+        (x2,y2)
c
c
c     area within dots represents felt region
c    '1' s represents line joining (x1,y1) (x2,y2)
c    phinc (long,lat) increment in degrees (in new coordinate
c       system) for which risk is to be computed
c
c  7. irow1, irow2, icol1, icol2   (free field)
c        starting and ending rows and columns for this run
c        accelerations computed for sites in these rows
c        and columns--may be subset of seismic felt region defined
c        in input 6.   Also irow1=irow2=icol1=icol2=0 permitted
c        if only individual sites on lines (input 8) are selected.
c  8. indv (free field)
c        number of line segments containing individual sites at
c        which acceleration is to be computed; zero if
c        only fixed grid is used.
c    If indv > 0 read next inputs, otherwise skip to input 9.
c     nvs (free field)
c       number of sites per line segment
c     xe1,ye1,xe2,ye2 (free field--indv lines, one pair per line)
c       end points (long, lat) of line segment:  sites will be
c       evenly spaced on line in new coordinate system
c    NOTE: NO SMOOTHING OF ACCELERATIONS AT SITES ON LINE FOR
c    ACCELERATIONS FROM EARTHQUAKES WITHIN AREAL SOURCE ZONES.
c    CALCULATIONS FOR UNIFORM SEISMICITY ONLY AT THESE SITES.
c  9. jent,mdis (free field)
c      jent=no of magnitudes for which acceleration is
c         tabulated as a function of distance   (maximum 8)
c      mdis=number of distances in attenuation table (maximum 20)
c  10. nam, tm (jent values of tm)
c      nam=identifier up to 10 characters-for example 'schn-seed'
c         identifies attenuation curves used
c      tm=magnitude for which table of distance versus acceleration
c         values follows  (free field)
c   *******magnitudes must be in descending order***
c  11. rtab(i), (atab(i,j),j=1,jent)   (mdis lines, i=1,mdis)
c     rtab(i)=ith tabular distance in kilometers from earthquake source
c     atab(i,j)=mean peak acceleration (or other ground motion parameter
c              at i th distance for j th magnitude.
c
c  Put area sources first, fault sources second
c
c    Area source inputs:
c
c    each area source - -
c  1a. srctyp, num, dumid (a1,1x,i2,1x,a40)
c      srctyp = a  for area sources
c             = f  for fault sources
c             = x  only character in last line of input file
c      num=0 for seismic source areas if no averaging for earthquake
c          location uncertainty is to be done at end of ground
c          motion computations for this source zone
c      num=98 for seismic source zones if averaging for earthquake
c           location uncertainty is to be done at end of computations
c           for this source zone
c      dumid = source name (up to 40 characters)
       
c  2a. yrnoc, iprint, als  (free format)
c
c      yrnoc=number of years over which the earthquake occurrences
c                       take place
c      iprint = -1  no statistics for this (intermediate) set of
c                occurrences
c             =1 statistical calculations and printout (on unit 16)
c             =2 same as =1 plus summary file for plot, etc (unit 2)
c             =3 omit printout; do summary file for plot (unit 2)
c      als = earthquake location uncertainty standard deviation in km;
c
c  3a. jseg,ifr,itot
c   jseg=num of pairs of quadrilatersl corner points in this source
c       set (jseg-1 quads).
c   seismicity to be apportioned by fractional area among itot sources.
c    ifr  = set number (ifr =1,2...itot in sequence)
c    itot=10 max;    total quad pairs for itot sources maximum 50
c  4a. jseg quadrilateral endpoint lines (two points per line):
c    quad corner points (left long, left lat), (right long, right lat),
c    in decimal degrees (if dsw=0)  degrees, minutes (if dsw=1)
c
c     x1,y1 ____________ x2,y2
c          1           1
c          1           1
c          1           1
c          1           1
c     x3,y3 ____________ x4,y4
c           1           1
c            1           1
c             1           1
c              1           1
c               1___________1
c         x5,y5              x6,y6
c
c   jseg=3 in this example:  quad endpoint pairs are
c                (x1,y1)---(x2,y2)
c                (x3,y3)---(x4,y4)
c                (x5,y5)---(x6,y6)
c    subregions of a set are defined as shown
c    subregions of one set are separate from those of other sets
c    repeat 3s, 4s for itot sources
c  5a. noc(l)  l=1,lev  (lev=12f6.2)   (lev=12 maximum)
c    numbers of events expected in yrnoc years in each
c          magnitude interval for earthquake occurrences
c  6a. fm(l), l=1,lev  (12f6.2)
c    fm(l)=center of magnitude interval for which noc(l) occur
c    If only one magnitude is given, magnitude interval = .5
c    (.25 higher and lower)
c
c     repeat 1s. thru 6s. for remaining sources
c
c        fault inputs:
c
c    each fault source - -
c  1f. srctyp, num, dumid (a1,1x,i2,1x,a40)
c
c      srctyp = f  for fault sources
c             = x  only character in last line of input file
c      num = 99 for first fault source
c      num = 00 for successive faults
c      dumid = source name (up to 40 characters)
c
c  2f. yrnoc, iprint, totl, als, bls, sigls (free format)
c
c      yrnoc=number of years over which the earthquake occurrences
c                       take place
c      iprint = -1  no statistics for this (intermediate) set of
c                occurrences
c             =1 statistical calculations and printout (on unit 16)
c             =2 same as=1 plus summary file for plot, etc (unit 2)
c             =3 omit printout; do summary file for plot (unit 2)
c      als,bls,sigls rupture length parameters
c      if als, bls non zero:
c      if als, bls non zero:
c        length=10**(als +bls*m +fr*sigl)
c        where m=magnitude;   sigl=standard deviation (in log length)
c    fr=(normally) 5 values in range (-2,+2)  if sigl non zero
c    fr=0 (1 value)--mean rupture length only if sigl=0
c    if als,bls,sigls=0. previous values are used
c    if no previous values input, default values are used:
c        als=-1.085    bls=.389   sigls=.52
c    totl=distance between faults if this set of faults is a set
c         of "dummy" faults  used to approximate a uniform field
c         of faults.
c     totl=0 if individual fault or a number of well defined faults
c         are used.
c     totl determines whether ground motions for a site are to
c         be smoothed in distance or smoothed in magnitude
c         (smoothed in magnitude if totl=0)
c         (smoothed in distance (totl nonzero).
c 3f. jseg, ifr, itot  (free field)
c    jseg=num of fault segment end points to be connected
c        into single fault   (jseg-1 segments)
c    ifr, itot as defined in 2s
c 4f. (xl(i,i(fr),yl(i,i(fr), i=1,jseg)   (free field)
c       xl=long  yl=lat (degrees)  (ifr=fault number)
c     jseg=24 max        itot=26 max
c 5f. noc(l)   same as in 5s.
c 6f. fm(l) same as 6s.
c    repeat 3f, 5f for itot faults
c
c    repeat 1f. thru 6f. for remaining faults
c
c    end with srctyp=x (omit other inputs on line)
c * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
      character  idsw*16,nam*10,tytle*80,dumid*40,srctyp*1
       data imaxsq/2500/
      common/cc/dumfac,fac(2500)
      dimension idsw(2),ibr(10),lsub(26),dtot(26)
      dimension xpt2(500),ypt2(500),xpt1(500),ypt1(500)
      dimension prls(10),rlls(10,25)
      dimension nrlsv(25),prlrl(10,25),rll(10,25)
      dimension disl(50),angl(50)
      common/ata/ata(20,25)
      common/src/srctyp
      common/inout/mdum(4),stsize
      common/slinds/sld1(4),perpa(4),sld2(4),sllq(4),
     & mina(4),perpaq(4),sldum2(8)
      common/extra/isid,shdis,distl(24),inear
      common /xlyl/xll(4),yl(4),xr(4),yr(4),xc(4),yc(4)
      common/itabs/dtab(105,25),maxa(25)
      common/magdis/jent,mdis
      dimension tmdif(10),atab(20,25),tm(10),atadif(20,25)
      common/rtb/rtab(20)
      common/xyarea/xsav(2,50),ysav(2,50),sarea(50)
c*******need maxdim=dimension of reg & of rawbin(maxdim,1600)
      data maxdim,maxar/55,1600/
      dimension regg(1600)
      dimension reg(55),rawbin(55,1600), smobin(55,1600)
      common/rdis/rdis
      common/wds/max,maxp,maxsp
      common/ext/prob,ex,exl
      common/tims/ftim(20),jtim(20),ntims,nwt,iprint
      data iscom/500/
c*** iscom = dimension of following sin,cos arrays
      common/sincos/sinx(500),cosx(500),siny(500),cosy(500)
      common/quad/maxq,ac(105),dumlog,aclog(105)
      dimension x(4),y(4),noc(24),fm(25)
      common/linps/dist(24,26),disq(24,26),jseg,jsegm,
     & xl(24,26),yp(24,26),coa(24,26),sia(24,26)
      common/lindis/dls(24),perp(24),min(24),persq(24)
      dimension jm(26), disrem(24)
      common/accel/gm(25),exdif(101),atah(20,25),aclim(2)
c arrays on previous line inserted for spreading out accelerations
      real noc
      integer dsw
      data linset,line,nrls/0,0,0/
      data irows,icols,indvpt/0,0,0/
      data idsw/'decimal degrees ','degrees, minutes'/
      data r,pi2,pi,pih/6378.,6.2831854,3.1415927,1.5707963/
      rad=pi/180.
c
      open(16,file='riskx.out',status='unknown')
c* * * * next line added by RL 3/94
      open(17,file='risk.table',status='unknown')
      open(3,file='unit.3',status='unknown')
      open(2,file='unit.2',status='unknown')
c
c     saved on unit 3 for later use or continuation
      rewind 3
      allerr=0.
      dlevsv=0.
      flevst=0.
      levno=0.
      tot5=0.
      read(5,20) tytle
   20 format(a80)
      write(16,30) tytle
   30 format(a80)
c  isw tells whether to include data from a previous run, zero if no.
c  sigmax=maximum value of sigma (km, in earthquake location)
      read(5,*)isw,sigmax
      if(isw.eq.0) write(16,40)
   40 format(' isw=0: new run--no previous results included')
      if(isw.eq.1) write (16,50)
   50 format(' isw=1: run continuation; add to previous results')
      if(isw.eq.0) go to 70
      idrc=1
   60 read(3,end=70) (smobin(iqrp,idrc),iqrp=1,maxdim)
      idrc=idrc+1
      go to 60
   70 rewind 3
c    prob=extreme probability in decimal
c    ntims are number of 'jtims' for which calculation is done
c    jtims are the durations for which the extreme motions are
c    to be calculated at the prob extreme probability level
      read (5,*) prob, ntims, (jtim(jv),jv=1,ntims)
      if (prob.lt.1.) go to 90
      write(16,80) prob
   80 format(' prob ='f8.3' must be decimal less than one')
      call exit
   90 continue
      write (16,100) prob,(jtim(jv),jv=1,ntims)
  100 format('   extreme probability 'f6.3
     &/'   for exposure times (years) '2(10i7/))
      ex=-alog(prob)
      exl=alog(ex)
      do 110 jv=1,ntims
  110 ftim(jv)=jtim(jv)
c
      read(5,*) scale,dsw,sd,inos
      write(16,120) scale,idsw(dsw+1),idsw(dsw+1),sd
  120 format(' scale factor for ground motion "box" levels='
     &f7.2/'    coordinates input in ' a16,
     &/'    coordinates are printed in 'a16,
     &/' variability in attenuation, sigma='f6.2)
c      scale=multiplicative factor for acceleration levels
c      dsw=0 if input is decimal degrees; =1 for degrees,minutes
c      sd=standard deviation for log acceleration
c   read (x1,y1), (x2,y2) =long,lat in dec. degrees of 2 points
c   to transform to equator
c      (x1,y1) to (0.,0)(x2,y2) to (dist. between points,0)
  130 read(5,*) x1,y1,x2,y2
      write (16,140) x1,y1,x2,y2
  140 format(' grid oriented parallel to great circle thru ('
     & f7.2','f6.2'),('f7.2','f6.2')')
c  convert to decimal degrees unless dsw=0.
      if(dsw.ne.0) call cond(x1,x2,y1,y2)
      x1=x1*rad
      x2=x2*rad
      y1=y1*rad
      y2=y2*rad
c   transform to equator
      call inieqr(x2,y2,x1,y1)
c
      call toeqr(x1,y1,x1t,y1t)
      call toeqr(x2,y2,x2t,y2t)
c  read in upper left and lower right corners of affected rectangle
c  -- degrees, minutes ie 20.30=20 degrees,30 minutes (dsw=1)
c      (decimal degrees 20.50 = 20 1/2 degrees (dsw=0) )
c  opposite corners of gridded area (in latitude and longitude)
c  gridded area becomes rectangle surrounding new equator.
c  sites are at uniform increments in latitude and longitude
c  within this gridded area in new coordinate system.
c  in order to limit affected area you can specify
c   beginning and ending rows and columns as irow1 and 2
c   and icol1 and 2 (next input line)
c      ph1=phi=latitude
c      fl1=lambda=longitude
c
      read(5,*) fl1,ph1,fl2,ph2,phinc
      write (16,160) fl1,ph1,fl2,ph2
  160 format(' corners of gridded area-upper left='f7.2','f7.2,
     &/23x' lower right='f7.2','f7.2)
c
c  convert to decimal degrees (unless dsw=0)
      if(dsw.eq.0) go to 170
      call cond(ph1,fl1,ph2,fl2)
      call condec(phinc)
  170 continue
c
c     determine number of subregions in map area and set up subscripting
c     irow=no of rows
c     icol=no of cols
      write(16,180) phinc,phinc
  180 format(' longitude increment='f7.4' (decimal degrees)'
     &/' latitude increment ='f7.4' (decimal degrees)')
      phinc=phinc*rad
      flinc=phinc
      dphinc=phinc*r
      dphinch=dphinc/2.
      dflinc=flinc*r
      fl1=fl1*rad
      ph1=ph1*rad
      fl2=fl2*rad
      ph2=ph2*rad
c    transform to equator and x-y rectangular coordinate system
c    convert to kilometers
      call toeqr(fl1,ph1,flam1,phi1)
      call to eqr(fl2,ph2,flam2,phi2)
      if(phi1.gt.0) go to 210
      if(phi2.gt.0) go to 220
c     apparent error in inputs or xformataion
  190 write(16,200)
  200 format(' probable input error--felt region upper left,'
     &' lower right'/' must be on opposite sides of new equator')
      call exit
  210  if(phi2.le.0) go to 230
      go to 190
  220 phs=phi1
      phi1=phi2
      phi2=phs
      fls=flam1
      flam1=flam2
      flam2=fls
c     have proper orientation--continue
  230 ph1=phi1
      if(flam1.lt.0) go to 240
      fl1=flam1
c    find total number of cols
      go to 260
  240 write(16,250)flam1,flam2
  250 format(' apparent error in xforming lat and long limits'
     & 'to eqr--'/'  flam1='f9.3'  fl2='f9.3)
      call exit
  260 continue
      ph1r=ph1*r
      phi2r=phi2*r
      fl1r=fl1*r
      flam2r=flam2*r
c   adjust to include area beyond edges for later smoothing
c     determine nbord=number of additional rows and columns at
c      which ground motions must be calculated to account for
c      earthquake location variability.
      nbord=2.*sigmax/dphinc+.01
      if(sigmax.eq.0.) go to 280
      if(nbord.ge.2) go to 280
      write (16,270) dphinc,sigmax,nbord
  270 format(' increments between sites ='e12.5' too large for'
     & 'sigmax='e12.5' nbord='i5)
      call exit
  280 ph1=ph1+nbord*phinc
      if (flam2.gt.fl1) flinc=-flinc
      fl1=fl1+nbord*flinc
      irow=(ph1-phi2)/phinc+1.01+nbord
      icol=(fl1-flam2)/flinc+1.01+nbord
      write(16,290)irow,icol,nbord
  290 format(' gridded region contains'i4,' rows, 'i4' cols',
     &' including border 'i4' rows and cols')
      read(5,*)irow1,irow2,icol1,icol2
c
c    limits rows and columns of felt points for which calculation is
c    to be made.
c
      write(16,300)irow1,irow2,icol1,icol2
  300 format(' for this run begin at row 'i3' end row 'i3
     & ', begin col 'i3' end col 'i3)
c   for earthquake location variability, compute at more sites
c   calculate at additional rows and cols
      irow2=irow2+2*nbord
      icol2=icol2+2*nbord
      icols=icol2-icol1+1
      irows=irow2-irow1+1
      if(irow1.eq.0) irows=0
      if(icol1.eq.0) icols=0
      if(irow1.le.irow2) go to 320
      write(16,310) irow1
  310 format(' irow1 ='i4'  too large--error stop')
      call exit
  320 if(icol1.le.icol2) go to 340
      write(16,330) icol1
  330 format(' icol1 ='i4'   too large--error stop')
      call exit
  340 if(icols*irows.le.maxar) go to 370
  350 write(16,360)irows,icols,indvpt,maxdim
  360 format(' irows='i4' icols='i4' pts on line='i4
     &' exceeds maxdim of 'i4)
      call exit
  370 continue
c   read in number of line segments containing sites at which
c   calculation is to be done  (in addition to or instead of
c   computing for sites on a fixed grid)
c   zero if no line segments
      uply=ph1*r
      uplx=fl1*r
      boty=phi2*r
      botx=flam2*r
      write(16,380) uplx,botx,uply,boty
  380 format(' new coordinates (km) gridded area'
     &/'       upper left='2f9.2'; lower right='2f9.2)
      read(5,*)indv
      if(indv.ne.0)write(16,390) indv
  390 format(' sites are also located on 'i3' line(s)')
      if (indv.eq.0) go to 460
c     calculate accelerations at nvs sites on each of indv lines
c     read in end point pairs (long,lat) xe1,ye1,xe2,ye2
c     for each of indv lines
c     nvs sites are evenly spaced on line*
c     nvs=2 gives accelerations at end points of line;
c     nvs=3 acceleration at 2 end points plus center of line,etc.
      read(5,*) nvs
      write(16,400) nvs
  400 format(i4' sites per line')
      indvpt=0
      do 450 i=1,indv
      read(5,*) xe1,ye1,xe2,ye2
      write(16,410) i,xe1,ye1,xe2,ye2
  410 format(' line 'i2' end points at 'f7.3','f7.3' and 'f7.3
     &','f7.3)
      if(dsw.ne.0) call cond(xe1,xe2,ye1,ye2)
      xe1=xe1*rad
      xe2=xe2*rad
      ye1=ye1*rad
      ye2=ye2*rad
      call toeqr(xe1,ye1,xo1,yo1)
      call toeqr(xe2,ye2,xo2,yo2)
      fr=0.
      if (nvs.gt.1) frinc=1./(nvs-1)
      do 440 iq=1,nvs
      indvpt=indvpt+1
      if(indvpt.le.500) go to 430
      write(16,420)
  420 format(' more than 500 single points requested as sites'
     & 1x,'do 500')
      go to 460
  430 xl1=xo1+fr*(xo2-xo1)
      yp1=yo1+fr*(yo2-yo1)
      cosy(1)=cos(yp1)
      siny(1)=sin(yp1)
      sinx(1)=sin(xl1)
      cosx(1)=cos(xl1)
      call backw(1,1,xpt1(indvpt),ypt1(indvpt))
      xpt2(indvpt)=xl1*r
      ypt2(indvpt)=yp1*r
      fr=fr+frinc
  440 continue
  450 continue
  460 if(icols*irows+indvpt.gt.maxar) go to 350
      nrc=icols*irows
c
c     jent=number of magnitude entries in table of acceleration as a
c        function of magnitude and distance
c     mdis=number of distance entries
      read(5,*) jent,mdis
      if (jent.le.10) go to 480
      write (16,470) jent
  470 format(' jent='i3' too large--max 10 allowed')
      call exit
  480 if (mdis.le.20) go to 500
      write(16,490) mdis
  490 format(' mdis='i4'  too large--max 20 distances allowed')
      call exit
  500 continue
c     read identifier and magnitudes for which acceleration versus
c     distance table follows.  magnitudes must be in decreasing order
c
      read(5,*)nam,(tm(i),i=1,jent)
c    print magnitude headings for attenuation function.
      write (16,510) nam
  510 format(/,' Attenuation function: 'a10/30x'Magnitude')
      write(16,520) (tm(i),i=1,jent)
  520 format(' dist(km) 'f6.2,10f8.2)
c  read in table of acceleration vs distance values for set
c  of magnitudes (distance increasing in table)
      do 580 m=1,mdis
      read(5,*) rtab(m),(atab(m,j),j=1,jent)
      if(atab(1,1).lt.10) go to 540
      write (16,530) rtab(m),(atab(m,j),j=1,jent)
  530 format(1h f10.2,10f10.2)
      go to 560
  540 write(16,550) rtab(m),(atab(m,j),j=1,jent)
  550 format(1h f8.2,10f8.4)
c  take logs of tabular values since interp. is in log acc. etc.
  560 do 570 j=1,jent
  570 atab(m,j)=alog(atab(m,j))
  580 continue
c
c      define limits for acceleration boxes:   accelerations in the
c      range ac(j-1) < ac < ac(j) will be collected the jth box.
c      max=number of boxes;  acceleration values too large for
c      any of the box levels correspond to maxp=max+1
c      as here constituted boxes have limits ranging from
c     .02 to 1.0
c     For motions exceeding 1(g)., provide
c     appropriate distance vs ground table and select 'scale'
c     for multiplying boxes to get consistent range
c     or rewrite and recompile subroutine BOX
  600 call box(scale,sd)
c
c     basic step size for distances in source areas
      do 610 i=1,maxsp
  610 aclog(i)=alog(ac(i))
c     number of digits after decimal for print of
c     accelerations (subroutine out) determined by scale
c
c     identify rows and cols for file02
  620 ir2=irow2-2*nbord
      icl2=icol2-2*nbord
      write(2,*) irow1,ir2,icol1,icl2,indvpt,ntims,
     &(jtim(jv),jv=1,ntims),prob,sd
c     create table of sines and cosines for later back transformation
c     of felt points
      if(icol1.eq.0) go to 680
      yp1=(ph1-(irow1-1.)*phinc)
      xl1=(fl1-(icol1-1.)*flinc)
      irow2p=irow2+1
      icol2p=icol2+1
      if(irow2p.le.iscom) go to 650
  630 write(16,640) irow2p,icol2p,iscom
  640 format (i5' rows,' i5' cols needed in sincos arrays,'
     &' dimension='i4/' need to increase dimension')
      call exit
  650 if(icol2p.gt.iscom) go to 630
      do 660 ira=irow1,irow2p
      cosy(ira)=cos(yp1)
      siny(ira)=sin(yp1)
  660 yp1=yp1-phinc
      do 670 ii=icol1,icol2p
      cosx(ii)=cos(xl1)
      sinx(ii)=sin(xl1)
  670 xl1=xl1-flinc
  680 read(5,691)srctyp,num,dumid
      if(srctyp.eq.'a')then
c       read(5,701)yrnoc,iprint,als
        read(5,*)yrnoc,iprint,als
        if(als.ne.0.)num=98
        if(als.eq.0.)num=00
      else if(srctyp.eq.'f')then
c       read(5,700)yrnoc,iprint,totl,als,bls,sigls
        read(5,*)yrnoc,iprint,totl,als,bls,sigls
      else if(srctyp.eq.'x')then
        num=99
        go to 2950
      endif
      numsav=num
  690 format('yrnoc= 'f6.0,' iprint='i2)
  700 format(f10.0,i2,f10.2,2f6.2,f5.2)
  701 format(f4.0,i2,f4.1)
c
      if(num.ne.99) go to 710
      if(line.eq.1) go to 2950
c  yrnoc=0 here if only source areas, no faults included
      if(yrnoc.eq.0.) go to 2950
      line=1
  710 write(16,692)dumid
      write(16,690)yrnoc,iprint
c     write(17,693)dumid
  691 format(a1,1x,i2,1x,a40)
  692 format(/,a40,/)
  693 format(a40)
  720 if(line.eq.1) go to 830
      if(num.eq.98) write (16,705) als
  705 format(' smooth for earthquake location uncertainty sigma='
     1 f5.1)
      nbr=0
      ist=0
      iend=0
c  read identifiers for source area inputs
  730 read(5,*)jseg,ifr,itot
c     write(16,740) ifr,itot
  740 format(' source 'i2' of 'i2)
c     jseg=number of pairs of quadrilateral end points in this
c        single source
c     itot=number of sources in this source area
c     ifr=identifies which source,  ifr=1,2,....itot
      if (itot.le.10) go to 760
      write(16,750) itot,jseg,ifr
  750 format('  itot ='i4'  to large--max 10  jseg='i4
     & ' ifr='i4)
      call exit
  760 ist=iend+1
      iend=ist+jseg-1
      if (iend.le.50) go to 780
      write(16,770)
  770 format('  too many quadrilaterals in source--max 50')
      call exit
c  read in boundaries for quadrilaterals for all subareas
c  in this source
c      read limits of seismic area x(i)=lambda(i),   y(i)=phi(i)
c      upper left(x,y) then  upper right(x,y) --1st quadrilateral
c      2nd line   lower left,lower right 1st quadrilateral=
c      2nd card   upper left,upper right   2nd quadrilateral,  etc.
c
  780 do 810 ii=ist,iend
      read(5,*)  (xsav(i,ii),ysav(i,ii), i=1,2)
  790 format(4f6.0)
  800 format(1h 4f10.3)
      write(16,800)(xsav(i,ii),ysav(i,ii),i=1,2)
  810 continue
      if(ifr.eq.itot) go to 820
      nbr=nbr+1
      ibr(nbr)=iend
      go to 730
  820 num=iend
      go to 960
c     fault line read
  830 j=1
      if(totl.ne.0.)write(16,840) totl
  840 format(' Distance between dummy faults='f5.1)
      dsum=0.
c    fault inputs
  850 read(5,*) jseg ,ifr,itot
      write(16,860) ifr,itot
  860 format(' fault 'i2' of 'i2)
c    jseg=number of end points of connected segments
c       for the current fault
c       jseg=one plus number of segments
c    itot=number of distinct faults in this zone
c    ifr identifies current fault, ifr=1,2,...itot
      if (jseg.le.24) go to 890
  870 write (16,880) jseg,itot
  880 format('  jseg='i4' (max 24) itot='i4' (max 26)-stop')
  890 if(itot.gt.26) go to 870
c    read in end points long,lat of each segment
      read(5,*)(xl(i,j),yp(i,j),i=1,jseg)
      write(16,900)(xl(i,j),yp(i,j),i=1,jseg)
  900 format(1h 4(f10.2,f8.2,','))
      jm(j)=jseg
      do 920 i=1,jseg
c    convert to decimal degrees unless dsw=0
      if(dsw.eq.0) go to 910
      call condec(yp(i,j))
      call condec(xl(i,j))
  910 continue
c   transform to equator--rectangular coordinates
      yin=yp(i,j)*rad
      xin=xl(i,j)*rad
      call to eqr(xin,yin,xout,yout)
      xl(i,j)=xout*r
      yp(i,j)=yout*r
  920 continue
c   compute line parameters, lengths for fault segments
      jsegm=jseg-1
      dtot(j)=0.
      do 930 i=1,jsegm
      xdelta=xl(i+1,j)-xl(i,j)
      ydelta=yp(i+1,j)-yp(i,j)
      disq(i,j)=xdelta*xdelta+ydelta*ydelta
      dist(i,j)=sqrt(disq(i,j))
      coa(i,j)=(xl(i+1,j)-xl(i,j))/dist(i,j)
      sia(i,j)=(yp(i+1,j)-yp(i,j))/dist(i,j)
  930 dtot(j)=dtot(j)+dist(i,j)
      dsum=dsum+dtot(j)
      if(ifr.eq.itot) go to 950
  940 j=j+1
      go to 850
c     read in num of occurrences for each magnitude (12f6.2)
c     both area and fault input
  950 jtot=j
  960 read (5,1170) (noc(i),i=1,12)
      lev=12
  970 if(noc(lev).ne.0.0) go to 990
      lev=lev-1
      if(lev.gt.0) go to 970
      write(16,980)
  980 format(' apparent input err for no of occurrences at each level')
      go to 2990
  990 continue
      write(16,1000)lev
 1000 format(' nr of levels of seismicity = 'i2)
c
c  read in corresponding mag. interval center points. (lev of them)
c
 1010 format(' before normalizing to rate/year')
 1020 format('    Earthquake rate / year')
      read(5,1170)(fm(i),i=1,lev )
c  magnitudes must be evenly spaced and in increasing order.
c  if not entered in increasing order, program reverses order
c  of magnitudes and corresponding earthquake occurrences
c  at each magnitude.
c  Don't confuse this with the original table of ground
c   motions as a function of magnitude and distance.
c   The magnitudes in that table must be read in decreasing
c   (rather than increasing) order.
      if(fm(1).lt.fm(2)) go to 1040
      if(lev.eq.1) go to 1040
      levh=lev/2
      ll=lev
      do 1030 l=1,levh
      sav=fm(l)
      fm(l)=fm(ll)
      fm(ll)=sav
      sav=noc(l)
      noc(l)=noc(ll)
      noc(ll)=sav
 1030 ll=ll-1
 1040 continue
      if(lev.gt.1) go to 1050
c    we cannot calculate beta from numbers of earthquakes
c    if only one level--so give a default beta so we
c    can later spread out
      beta=-2.0
      bval= 2.0*.43429
      fm(2)=fm(1)+.5
      delm=.5
      go to 1060
 1050 continue
      dlev=abs(fm(1)-fm(2))
      beta=alog( noc(2)/noc(1) )/dlev
      bval=-beta*.43429
 1060 write(16,1070) bval
 1070 format('b-value ='f8.4)
      if(yrnoc. eq. 1.) go to 1090
      write(16,1010)
      write(16,1100) (noc(ll),ll=1,lev)
c     this is a normalization to rate per year
      do 1080 lm=1,lev
      noc(lm)=noc(lm)/yrnoc
 1080 continue
 1090 write(16,1020)
      write(16,1100)(noc(l),l=1,lev)
 1100 format(' occurrences='12f10.6)
      write(16,1110)(fm(l),l=1,lev)
 1110 format('  magnitudes='12f10.2)
      summoc=0.
      do 1120 lm=1,lev
 1120 summoc=summoc+noc(lm)
c     evaluate at twice as many magnitudes as initially read
c     in if inos=1
      if(inos.ne.1)  go to 1140
      dlev=(fm(2)-fm(1))/2.
      zac=exp(abs(beta)*dlev)
      noc(1)=noc(1)*zac/(1.+zac)
      summoc2=noc(1)
      lev=2*lev
      fm(1)=fm(1)-dlev/2.
      do 1130 ll=2,lev
      fm(ll)=fm(ll-1)+dlev
      noc(ll)=noc(ll-1)/zac
 1130 summoc2=summoc2+noc(ll)
      write(16,1150) summoc,summoc2
      write(16,1100) (noc(l),l=1,lev)
      write(16,1110) (fm(l),l=1,lev)
 1140 continue
 1150 format(' total eqs='2e12.5)
 1160 continue
 1170 format(12f6.2)
c
      if (line.eq.0) go to 1250
c    determine which rupture length magnitude relationship to use
      if(als.eq.0)   go to 1240
      al=als
      bl=bls
      sigl=sigls
c     compute break length
 1180 write(16,1190) al,bl,sigl
 1190 format(' fault rupture length parameters al='f7.3' bl='f7.3
     &'  sigl='f6.2)
      linset=1
      if (sigl.gt.0) go to 1230
c     which single break default do we want
c     if bl=0, al non zero use old Algermissen-Perkins default values
c     use parameter values supplied for one break only
 1200 do 1210 i=1,lev
 1210 rlls(1,i)=10.**(al+bl*fm(i))
 1220 prls(1)=1.
      nrls=1
      go to 1350
c    set up rupture lengths and probabilities as fct of magnitude
c    nrls lengths per magnitude
c    al, bl, sigl non zero
 1230 call pbreak(fm,al,bl,sigl,rlls,prls,lev,nrls)
      go to 1350
 1240 if (nrls.eq.1) go to 1200
      if(linset.eq.1) go to 1230
c     compute default values
      al=-1.085
      bl=.389
      sigl=.52
      linset=1
      go to 1180
c  convert quad corner points to decimal degrees  (if dsw =1)
c
 1250 do 1280 ii=1,num
      do 1280 i=1,2
      if(dsw.eq.0) go to 1260
      call condec(xsav(i,ii))
      call condec(ysav(i,ii))
 1260 continue
      xin=xsav(i,ii)*rad
      yin=ysav(i,ii)*rad
      call to eqr(xin,yin,xout,yout)
      xsav(i,ii)=xout*r
      ysav(i,ii)=yout*r
c     write(16,1270) xsav(i,ii),ysav(i,ii),ii
 1270 format(' xsav,ysav='2f9.3' ii='i3)
 1280 continue
c
 1290 nm=num-1
      stot=0.
      do 1330 ii=1,nm
      if(nbr.eq.0) go to 1310
      do 1300 iq=1,nbr
      if(ii.eq.ibr(iq)) go to 1330
 1300 continue
c    set line parameters for subsource
 1310 call setreg(ii)
      do 1320 i=1,2
      x(i)=xsav(i,ii)
      x(i+2)=xsav(i,ii+1)
      y(i)=ysav(i,ii)
 1320 y(i+2)=ysav(i,ii+1)
c    find area of subregion ii
      s1=abs(x(1)*(y(2)-y(3))+x(2)*(y(3)-y(1))+x(3)*(y(1)-y(2)))
      s2=abs(x(4)*(y(2)-y(3))+x(2)*(y(3)-y(4))+x(3)*(y(4)-y(2)))
      sarea(ii)= (s1+s2)/2.
      stot=stot+sarea(ii)
 1330 continue
      eua=noc(1)
c   find earthquake rate per unit area for lowest magnitude
c   rate will be printed but not used in program
      delm=(fm(2)-fm(1))/2.
      fm1=fm(1)-delm
      fm2=fm(1)+delm
      eua=eua/stot
      write(16,1340) stot,eua,fm1,fm2
 1340 format(' Area='f9.0' sq km, rate/sq km='e12.5,
     & ' for mags 'f5.2'-'f5.2)
 1350 dlev=fm(1)-fm(2)
c    create table of log accelerations for set of magnitudes
c    in (tm) for the original set of mdis distances as a function
c    of distance
c    next three lines test whether table has already
c     been computed--if so skip over
      if(lev.gt.levno) go to 1360
      if(fm(1).ne.flevst) go to 1360
      if(dlev.eq.dlevsv) go to 1520
 1360 do 1370 ill=2,jent
 1370 tmdif(ill)=tm(ill)-tm(ill-1)
      do 1420 ll=1,lev
      do 1380 il=2,jent
      if(fm(ll).gt.tm(il)) go to 1390
 1380 continue
      il=jent
 1390 ilm=il-1
      do 1400 j=1,mdis
 1400 ata(j,ll)=atab(j,ilm)+(fm(ll)-tm(ilm))/tmdif(il)
     & *(atab(j,il)-atab(j,ilm))
      do 1410 j=2,mdis
 1410 atadif(j,ll)=ata(j,ll)-ata(j-1,ll)
 1420 lsub(ll)=il
c     find distance corresponding to each acceleration in
c     table ac (set up in subroutine box) for each magnitude
      do 1510 l=1,lev
      do 1500 j=1,maxp
      do 1430 i=2,mdis
      if(aclog(j).gt.ata(i,l)) go to 1450
 1430 continue
c     beyond table
 1440 dtab(j,l)=rtab(mdis)
      go to 1500
 1450 if(atadif(i,l).ne.0) go to 1490
c     no change in acceleration at this distance
 1460 if(i.eq.2) go to 1480
      write(16,1470) i,l,j
 1470 format(' apparent error indist-table i,l,j='3i4)
      call exit
 1480 dtab(j,l)=0.
      go to 1500
 1490 dtab(j,l)=rtab(i)-(rtab(i)-rtab(i-1))*(ata(i,l)-aclog(j))
     &/atadif(i,l)
      if(dtab(j,l).lt.0) dtab(j,l)=0.
 1500 continue
 1510 continue
      dlevsv=dlev
      levno=lev
      flevst=fm(1)
 1520 continue
c    add half values of mag for later spreading out
      dmag=fm(2)-fm(1)
      bmag=fm(1)-dmag/2.
      levp=lev+1
      do 1530 i=1,levp
      gm(i)=bmag
 1530 bmag=bmag+dmag
      do 1570 ll=1,levp
      do 1540 il=2,jent
      if(gm(ll).gt.tm(il)) go to 1550
 1540 continue
      il=jent
 1550 do 1560 j=1,mdis
      atah(j,ll)=atab(j,il-1)+(gm(ll)-tm(il-1))/tmdif(il)
     & *(atab(j,il)-atab(j,il-1))
 1560 continue
 1570 continue
c    take exp differences for later accel spread out
c    the "exdif" values are used in subroutine "spread"
      if (lev.eq.1) betad=2.0
      if(lev.ne.1) betad=alog(noc(1)/noc(2))
      beta=betad/dmag
      fmul=1./(1.-exp(-beta*dmag))
      frak=exp(-beta*dmag/100.)
      frm=frak
      exdif(1)=0.
      do 1580 i=2,101
      exdif(i)=(1.-frm)*fmul
 1580 frm=frm*frak
      dtmax=dtab(1,1)
      if(dlev.lt.0) dtmax=dtab(1,lev)
c     find entry for each magnitude in distance-acceleration table
c     for zero distance (highest acceleration for that magnitude)
      do 1600 l=1,lev
      j=maxp
 1590 if(dtab(j,l).ne.0.) go to 1600
      j=j-1
      go to 1590
 1600 maxa(l)=j
c
      irow1s=irow1
      irow2s=irow2
      icol1s=icol1
      icol2s=icol2
      idrc=0
      do 2770 ispt=1,2
      if(ispt.eq.1) go to 1610
c    case of calculation done for single points on line
      if(indvpt.eq.0) go to 2770
      irow1s=1
      irow2s=1
      icol1s=1
      icol2s=indvpt
      go to 1620
 1610 if(irow1.eq.0) go to 2770
 1620 do 2760 ira=irow1s,irow2s
c
      ymid=(ph1-(ira-1.)*phinc)*r
      do 2750 ii=icol1s,icol2s
c
      idrc=idrc+1
      if(ispt.eq.2) go to 1630
      fii=ii
      xmid=(fl1-(fii-1.)*flinc)*r
      go to 1640
 1630 xmid=xpt2(ii)
      ymid=ypt2(ii)
 1640 if(isw.eq.0) go to 1660
c
c    after first time thru, read accelerations calculated
c    for previous sources at this site (saved in 2 dimensional
c    array rawbin) into reg (1 dimensional) array.
c
      do 1650 iqrp=1,maxdim
 1650 reg(iqrp)=rawbin(iqrp,idrc)
      go to 1680
c
c   first time thru clear reg array if not continuation run
c
 1660 do 1670 ll=1,maxp
 1670 reg(ll)=0.
      regg(idrc)=9999.999
c    set regg to min dist from fault
c
c    for fixed  affected area in row ira, col ii
c
c      source area computation
 1680 if(line.eq.1) go to 2120
      do 2110 k=1,nm
      if(nbr.eq.0) go to 1700
      do 1690 iq=1,nbr
      if(k.eq.ibr(iq)) go to 2110
 1690 continue
 1700 approx=0.
      do 1710 i=1,2
      xll(i)=xsav(i,k)
      xll(i+2)=xsav(i,k+1)
      yl(i)=ysav(i,k)
      yl(i+2)=ysav(i,k+1)
 1710 continue
      xr(3)=xll(1)
      yr(3)=yl(1)
      xr(1)=xll(2)
      yr(1)=yl(2)
      xr(2)=xll(4)
      yr(2)=yl(4)
      xr(4)=xll(3)
      yr(4)=yl(3)
      aread=sarea(k)/stot
c     determine whether site is inside or outside k th quadrilateral
c     subregion.  find distance rc to closest endpoint
c     and rf to farthest end point.  Set up table of distance versus
c     arc length for distances in the range rc to rf using
c     subroutines outsid or inside.
c      itst=no of radii at fixed increments between rc and rf
c           at which arc lengths are to be evaluated in table
c      itst=7   set in subroutine rrisk--recompile to change
      call rrisk(xmid,ymid,in,k,dis,rc,rf,itst)
      if(in.eq.1) go to 1740
      if(rc.lt.dtmax) go to 1730
      do 1720 ll=1,lev
 1720 reg(1)=reg(1)+aread*noc(ll)
      go to 2110
 1730 rcs=rc
      disl(1)=rc
      angl(1)=0.
      itst=itst+1
      ibeg=2
      go to 1750
 1740 disl(1)=0.
      angl(1)=0
      disl(2)=rc
      angl(2)=pi2*rc
      ibeg=3
c****next 4 lines changed Sept 8, 1988
c***replaces single line "if (rc.eq.0.) ibeg=2"
      if(rc.gt.0.) go to 1745
      ibeg=2
      itst=itst-1
 1745 rcs=0.
      itst=itst+2
c   compute distance versus arc length table for even increments
c   in distance
 1750 do 1810 iii=ibeg,itst
      if(in.eq.0.) go to 1760
      call inside(xmid,ymid,k,dis,angle)
      go to 1780
 1760 call outsid(xmid,ymid,k,dis,angle,kerr)
 1770 format('err outside for ira='i4' ii='i4)
 1780 continue
      disl(iii)=dis
      angl(iii)=angle*dis
      if(kerr.eq.0) go to 1810
      write (16,1770) ira,ii
      do 1790 imkl=1,iii
 1790 write(16,1800) disl(imkl),angl(imkl)
 1800 format(' dist, ang='2e12.5)
 1810 dis=dis+stsize
      itst=itst+1
      disl(itst)=rf
      angl(itst)=0.
c   compute distance versus arc length table for distances
c   to selected vertices and perpendiculars to edges
      rcsq=rc*rc
      rfsq=rf*rf
      do 1940 i=1,4
      ire=0
      if(in.eq.0) ire=2
      if(sllq(i).le.rcsq+1.) go to 1930
      if(sllq(i).ge.rfsq-1.) go to 1930
      dis=sqrt(sllq(i))-.001
      if(dis.le.rc) goto 1930
c    add angle, distance corresponding to vertex
      if (in.eq.0) go to 1830
      call inside(xmid,ymid,k,dis,angle)
      go to 1860
 1820 if(mina(i).ne.3) go to 1940
c   closest side next
      dis=abs(perpa(i))
      if(dis.le.rc+.5) go to 1940
      if(dis.ge.rf-.5) go to 1940
      dis=dis-.001
      call inside(xmid,ymid,k,dis,angle)
      ire=1
      go to 1860
 1830 continue
      call outsid(xmid,ymid,k,dis,angle,kerr)
      go to1860
 1840 if(mina(i).ne.3) go to 1940
      dis=abs(perpa(i))
      ire=1
      do 1850 iql=1,4
      if(abs(sllq(iql)-perpaq(i)).le. 2.) go to 1940
 1850 continue
      if(dis.le.rc+1.) go to 1940
      if(dis.ge.rf-1.) go to 1940
      dis=dis-.001
      call outsid(xmid,ymid,k,dis,angle,kerr)
 1860 continue
      if(kerr.eq.0) go to 1880
      write (16,1770) ira,ii
      do 1870 imkl=1,iii
 1870 write(16,1800) disl(imkl),angl(imkl)
 1880 continue
      do 1890 ll=1,itst
      if(dis-disl(ll)) 1900,1920,1890
 1890 continue
c   insert new distance, angle
 1900 jk=itst
      do 1910 mm=ll,itst
      angl(jk+1)=angl(jk)
      disl(jk+1)=disl(jk)
 1910 jk=jk-1
      disl(ll)=dis
      angl(ll)=angle*dis
      itst=itst+1
 1920 if(ire.eq.0) go to 1820
      if(ire.eq.2) go to 1840
      go to1940
 1930 if(in)1840,1840,1820
 1940 continue
      itst=itst+1
      disl(itst)=rf
      angl(itst)=0.
c   for each magnitude, source, determine fractional source area giving
c   acceleration a:  ac(i-1) < a < ac(i)  for each entry in acceleration
c   table ac.  equivalent to finding fractional source area at distance
c   from site between successive entries in acceleration-distance
c   table:  dtab(i,l) < d < dtab(i-1,l) for magnitude l.
c   interpolates for distance is disl vs angl table
c   area=integral(angle*dist*delta(dist))  from dtab(i-1,l) to dtab(i,l)
      do 2090 ll=1,lev
      stnoc=noc(ll)/stot
      iswd=0
      maxm=maxa(ll)
      if(maxm.le.0.) go to 1960
      approx=0.
      if(in.eq.0) go to 1950
      jlo=maxm
      go to 1980
 1950 if(rcs.lt.dtab(1,ll)) go to 1970
 1960 reg(1)=reg(1)+aread*noc(ll)
      go to 2080
 1970 jlo=indpt(dtab(1,ll),rcs,maxm)
 1980 j=jlo
      iqs=2
      do 2060 jk=1,jlo
      if(jk.gt.1) go to 1990
      dbot=rcs
      ddis=dtab(j,ll)
      if(dtab(j,ll).lt.rf) goto 2010
      reg(j+1)=reg(j+1)+sarea(k)*stnoc
      go to 2080
 1990 if(dtab(j,ll).lt.rf) go to 2000
      dbot=dtab(j+1,ll)
      ddis=rf
      iswd=1
      go to 2010
 2000 ddis=dtab(j,ll)
      dbot=dtab(j+1,ll)
 2010 anarea=0.
      do 2020 iq=iqs,itst
      if(dbot.le.disl(iq)) go to 2030
 2020 continue
      iq=itst
c    distance exceeds last value-error since table set up to handle this
 2030 dtop=disl(iq)
      if(disl(iq).gt.ddis) dtop=ddis
      dmid=(dtop+dbot)/2.
      delr=dtop-dbot
      fr=(dmid-disl(iq-1))/(disl(iq)-disl(iq-1))
      ang=angl(iq-1)+(angl(iq)-angl(iq-1))*fr
      anarea=anarea+ang*delr
      if(dtop.eq.ddis) go to 2050
      dbot=dtop
      iq=iq+1
      if(iq.le.itst) go to 2030
      write(16,2040)
 2040 format(' err in arc area computation')
      go to 2030
c    interpolate in angle , distance table
 2050 approx=approx+anarea
      reg(j+1)=reg(j+1)+anarea*stnoc
      j=j-1
      if(iswd.eq.0) go to 2060
      arerr=(approx-sarea(k))/sarea(k)
      allerr=allerr+1.
      if (abs(arerr).lt. .05) go to 2090
      tot5=tot5+1.
      if(abs(arerr).le. .10) go to 2090
      write(16,2100) arerr,ira,ii,rc,rf,k,ll,in,dumid
      go to 2090
 2060 iqs=iq
c     need to add remaining area if any to lowest acceleration
      dif=sarea(k)-approx
      if(dif.lt.0) go to 2080
 2070 reg(1)=reg(1)+dif*stnoc
 2080 approx=sarea(k)
 2090 continue
 2100 format(' int err='f6.3' row 'i2' col 'i2' rc='
     & f7.2' rf='f7.2' k='i2' ll='i2' in='i1,1x,a20)
 2110 continue
      go to 2730
c     fault computation
 2120 do 2720 j=1,jtot
      jseg=jm(j)
      jsegm=jseg-1
      call cldis(xmid,ymid,j)
      do 2160 ll=1,lev
c     fill in rupture lengths--make  certain that rupture length
c     does not exceed total fault length
      nrl=nrls
      do 2130 irl=1,nrl
      rll(irl,ll)=rlls(irl,ll)
 2130 prlrl(irl,ll)=prls(irl)
      if(rll(1,ll).lt.dtot(j)) go to 2160
      rll(1,ll)=dtot(j)
      if(nrl.eq.1) go to 2160
 2140 if(rll(2,ll).lt.dtot(j)) go to 2160
      prlrl(1,ll)=prlrl(1,ll)+prlrl(2,ll)
      nrl=nrl-1
      if(nrl.eq.1) go to 2160
      do 2150 i=2,nrl
      prlrl(i,ll)=prlrl(i+1,ll)
 2150 rll(i,ll)=rll(i+1,ll)
      go to 2140
 2160 nrlsv(ll)=nrl
      pt1=sqrt(shdis)
c     if(pt1.lt.regg(idrc)) regg(idrc)=pt1
      regg(idrc)=pt1
      if (isid.eq.0) go to 2490
c    distance is monotonically increasing for all segments
c    (the decreasing case has been converted to increasing)
c    or single shortest distance is interior to fault
      do 2480 ll=1,lev
      nrl=nrlsv(ll)
      frcon=dtot(j)/dsum*noc(ll)
      maxm=maxa(ll)
      jlo=indpt(dtab(1,ll),pt1,maxm)
      if (jlo.gt.0) go to 2170
c   distance beyond table--lump acceleration in first box
      reg(1)=reg(1)+frcon
      go to 2480
 2170 continue
      if (isid.eq.1) go to 2190
 2180 ibeg=1
      ifin=jsegm
      ixt=1
      dnear=dtot(j)
      inear=jsegm
      go to 2310
 2190 ibeg=1
      ifin=inear
      ixt=2
c   dnear=distance along fault from one end to point on fault
c   closest to the site
c   dnear=total length of segments 1,2,...inear
c   inear defined sub cldis--in common/extra/
      dnear=0.
      do 2200 iq=1,inear
 2200 dnear=dnear+distl(iq)
      dfar=dtot(j)-dnear
      totac=0.
      do 2220 irl=1,nrl
      smd=rll(irl,ll)
      if(smd.lt.dtot(j)) go to 2210
c     break is entire segment
      totac=totac+frcon*prlrl(irl,ll)
      sumr=totac
      go to 2220
 2210 smd2=smd/2.
c    find fraction of fault at closest distance to site
c    for interior site
      arcn=smd2
      arcf=smd2
      if(dnear.lt.smd) arcn=dnear-smd2
      if(dfar.lt.smd) arcf=dfar-smd2
      arct=arcn+arcf
      if(arct.lt. .0001) go to 2220
      totac=totac+frcon/(dtot(j)-smd)*arct*prlrl(irl,ll)
 2220 continue
      sumr=totac
      totacs=totacs+totac
      jlop=jlo+1
      if(totl.gt.0) go to 2230
c     spread dummy faults over "totl" km distance if "totl" > 0
c      if "totl" = 0, spread over magnitude instead
      call spread(reg,jlop,ll,totac,pt1)
      go to 2310
 2230 width=totl/2.
      sumr=sumr/totl
      add=0
      if(abs(pt1).gt.width) go to 2280
      qtot=width+abs(pt1)
      dsvs=0.
      jlo2=maxm
      do 2270 iqk=1,2
      jrp=jlo2
      do 2250 jk=1,jlo2
      dl3 =dtab(jrp,ll)
      if(dl3.gt.qtot) go to 2260
      cary=dl3-dsvs
      reg(jrp+1)=reg(jrp+1)+sumr*cary
      add=add+sumr*cary
      jrp=jrp-1
 2240 format(' add='e12.5)
 2250 dsvs=dl3
      jrp=0
 2260 cary=qtot-dsvs
      add=add+sumr*cary
      jrpp=jrp+1
      reg(jrp+1)=reg(jrp+1)+sumr*cary
      dsvs=0.
      qtot=width-abs(pt1)
 2270 jrp=jlo2
      jrpp=jrp+1
      go to 2310
 2280  dstart=abs(pt1)-width
      qtot=totl+dstart
      jlo2=indpt(dtab(1,ll),dstart,maxm)
      dsvs=dstart
      jrp=jlo2
      do 2290 jk=1,jlo2
      dl3=dtab(jrp,ll)
      if(dl3.gt.qtot) go to 2300
      cary=dl3-dsvs
      reg(jrp+1)=reg(jrp+1)+sumr*cary
      add=add+sumr*cary
      jrp=jrp-1
 2290 dsvs=dl3
      jrp=0
 2300 cary=qtot-dsvs
      reg(jrp+1)=reg(jrp+1)+cary*sumr
c   find frac of fault in distance range dtab(jr,ll) < d <dtab(jr-1,ll)
c   for each magnitude ll
 2310 do 2470 ithru=1,ixt
      jr=jlo
      ilo=1
      iii=ibeg
      casum=0.
      irl=1
      dsav=-dls(iii)
 2320 frac=0.
      jk=1
      smd=rll(irl,ll)
      if(smd.lt.dtot(j)) go to 2330
      if(ixt.eq.2)go to 2450
c    add in entire fault length
      totac=frcon*prlrl(irl,ll)
      jlop=jlo+1
      call spread(reg,jlop,ll,totac,pt1)
      go to 2450
 2330 cuml=dnear-smd
      if(cuml.le.0) go to 2450
      do 2340 iq=irl,nrl
 2340 frac=frac+prlrl(iq,ll)/(dtot(j)-rll(iq,ll))
      frac=frac*frcon
      dlen=0.
      do 2350 iq=ibeg,ifin
      disrem(iq)=distl(iq)
      dlen=dlen+disrem(iq)
      if (dlen.ge.cuml) go to 2370
 2350 continue
      write(16,2360)dlen,ibeg,ifin,cuml
 2360 format(' error for fault line dlen='e12.5' ibeg='i3
     &' ifin='i3' cuml='e12.5)
      call exit
 2370 disrem(iq)=cuml-dlen+disrem(iq)
 2380 ifins=iq
 2390 dend=disrem(iii)-dls(iii)
c     dsav=starting distance along segment iii
 2400 continue
      do 2410 jk=ilo,jlo
      dl3=sqrt(dtab(jr,ll)**2-persq(iii))
      if(dl3.gt.dend) go to 2440
      carea=dl3-dsav
      casum=casum+carea
      reg(jr+1)=reg(jr+1)+carea*frac
      jr=jr-1
 2410 dsav=dl3
c   at top of table--remaining dist beyond table
 2420 do 2430 iq=irl,nrl
      cuml=dnear-rll(iq,ll)
 2430 reg(1)=reg(1)+(cuml-casum)*prlrl(iq,ll)*frcon
     & /(dtot(j)-rll(iq,ll))
      go to 2460
 2440 carea=dend-dsav
      casum=casum+carea
      reg(jr+1)=reg(jr+1)+carea*frac
      dsav=dend
c      at end of segment iii
      if(iii.eq.ifins)  go to 2450
      iii=iii+1
      dsav=-dls(iii)
      ilo=jk
      go to 2390
 2450 irl=irl+1
      ilo=jk
      if(irl.le.nrl) go to 2320
      if(ithru.eq.ixt) go to2480
 2460 irl=1
      ibeg=inear+1
      ifin=jsegm
      dnear=dtot(j)-dnear
 2470 continue
 2480 continue
      go to 2720
c     more than one turning point
c     brute force.  find contribution for single break
c     increment break centers by distance deltal along fault from
c     one end to other.
 2490 continue
      deltal=10.
      if(pt1.gt.100) deltal=20.
      if(pt1.lt.15.) deltal=5.
      do 2710 ll=1,lev
      nrl=nrlsv(ll)
      do 2710 irl=1,nrl
      smd=rll(irl,ll)
      prnoc=noc(ll)*prlrl(irl,ll)
      dj=dtot(j)-smd
      nd=dj/deltal+1.5
      if(nd.gt.1) go to 2500
      nd=2
      if(dj.gt.1.) go to 2500
c     break is entire fault length
      maxm=maxa(ll)
      jlo=indpt(dtab(1,ll),pt1,maxm)
      add=prnoc*dtot(j)/dsum
      jlop=jlo+1
      call spread(reg,jlop,ll,add,pt1)
      go to 2710
 2500 continue
      dmvp=prnoc*dtot(j)/(dsum*(nd-1))
      del=dj/(nd-1)
      do 2700 m=1,nd
      dst=(m-1)*del
      do 2510 i=1,jsegm
      if(dst.lt.dist(i,j)) go to 2530
      dst=dst-dist(i,j)
 2510 continue
      write(16,2520)dst,j
 2520 format(' apparent error fault dist--dst='e12.5'  j='i4)
      go to 2990
 2530 ddsav=1.e8
      dfin=dst+smd
 2540 dover=0.
      if(dfin.le.dist(i,j)) go to 2550
      dover=dfin-dist(i,j)
      dfin=dist(i,j)
 2550 if(dls(i).lt.dst) go to 2560
      if(dls(i).gt.dfin) go to 2570
c      shortest distance to segment at point within segment
      dd=persq(i)
      go to 2580
c      shortest dist at start of segment
 2560 dd=(dls(i)-dst)**2+persq(i)
      go to 2580
c      shortest distance at end of segment
 2570 dd=(dls(i)-dfin)**2+persq(i)
 2580 continue
      if(dd.lt.ddsav) ddsav=dd
      if(abs(dover).lt.0.001) go to 2600
      dst=0
      i=i+1
      dfin=dover
      if(i.le.jsegm) go to 2540
 2590 format(' i too large i='i2)
      write(16,2590) i
      call exit
 2600 dis=sqrt(ddsav)
c  have distance to subsegment of arc of fault
c
      if(m.gt.1) go to 2610
      jen2=indpt(dtab(1,ll),dis,maxa(ll))
      disav=dis
      go to 2700
 2610 disdif=dis-disav
      if(disdif.eq.0) go to 2620
      dfac=dmvp/disdif
c    spread out contributions into accel boxes
      if(disdif) 2630,2620,2660
 2620 jlop=jen2+1
      call spread(reg,jlop,ll,dmvp,dis)
      go to 2700
c     distance decreasing
 2630 jen2=jen2+1
 2640 distab=dis-dtab(jen2,ll)
      if (distab.lt.0) go to 2650
      reg(jen2)=reg(jen2)+(dis-disav)*dfac
      add=(dis-disav)*dfac
      if(add.lt.0) print *,2740
      jen2=jen2-1
      disav=dis
      go to 2700
 2650 reg(jen2)=reg(jen2)+(dtab(jen2,ll)-disav)*dfac
      add=(dtab(jen2,ll)-disav)*dfac
      if(add.lt.0) print*,2750
      disav=dtab(jen2,ll)
      jen2=jen2+1
      go to 2640
 2660 if(jen2.eq.0) go to 2680
 2670 distab=dis-dtab(jen2,ll)
      if(distab.gt.0) goto 2690
 2680 reg(jen2+1)=reg(jen2+1)+(dis-disav)*dfac
      add=(dis-disav)*dfac
      if(add.lt.0) print *, 2780
      disav=dis
      go to 2700
 2690 reg(jen2+1)=reg(jen2+1)+(dtab(jen2,ll)-disav)*dfac
      add=(dtab(jen2,ll)-disav)*dfac
      if(add.lt.0) print*, 2790
      disav=dtab(jen2,ll)
      jen2=jen2-1
      if (jen2.ge.1) go to 2670
      go to 2680
 2700 continue
 2710 continue
c    above ends loop for more than one turning point
 2720 continue
 2730 continue
      do 2740 iqr=1,maxdim
 2740 rawbin(iqr,idrc)=reg(iqr)
c     save accelerations for this source & site in rawbin array
c
c    ends loop for one site, one area or fault
c    repeat for all sites for this source before going on to next
c    source
 2750 continue
 2760 continue
c    one source completed for all sites
 2770 continue
c----smooth here if sigma for earthquake locations has changed
c----from previous value
      if(line.eq.1) go to 2850
c-----no smoothing for faults
      if(numsav.ne.98) go to 2850
c   if sigma=0, no smoothing
      if(als.eq.0) go to 2850
       if(nrc.eq.0) go to 2850
c-----
      ismsw=1
       sigd=(dphinc/als)**2
      do 2780 im=1,imaxsq
 2780 fac(im)=exp((-im*sigd)/2.)
      kmax=10
      eqsum=0.
      dumfac=1.
c    compute icon based on sigma and dphinc
c    probably enough to go out 2 sigma on either side of point
c    find how many rows or columns that corresponds to
      icon=2*als/dphinc+1.01
      if(icon.lt.5) icon=5
c   if icon<5, set icon=5
      irm=irow2-nbord
      icm=icol2-nbord
      isub1=irow1+nbord
      isub2=icol1+nbord
      do 2830 iia=isub2,icm
      j1=iia-icon
      j2=iia+icon
      if(j2.gt.icol2) j2=icol2
      if(j1.lt.icol1) j1=icol1
      do 2830 iraa=isub1,irm
      i1=iraa-icon
      i2=iraa+icon
      if(i1.lt.irow1) i1=irow1
      i2=iraa+icon
      if(i2.gt.irow2) i2=irow2
      do 2800 iq=1,maxp
      add=0.
      wtt=0.
      do 2790 i=i1,i2
      do 2790 j=j1,j2
      ksq=(iraa-i)**2+(iia-j)**2
      if(ksq.gt.imaxsq) go to 2790
c     we need to convert to proper subscript in rawbin
c      in rawbin, first all cols for one row
c      (i-1)*icol(s)+j=row and col with single subscript in rawbin
      isub=(i-irow1)*icols+j-icol1+1
      add=add+fac(ksq)*rawbin(iq,isub)
      wtt=wtt+fac(ksq)
 2790 continue
      reg(iq)=0
      if(wtt.ne.0) reg(iq)=add/wtt
      eqsum=eqsum+reg(iq)
 2800 continue
 2810 continue
      isub=(iraa-irow1)*icols+iia-icol1+1
      do 2820 iq=1,maxdim
 2820 smobin(iq,isub)=reg(iq)+smobin(iq,isub)
 2830 continue
c   now delete entries from rawbin array
      do 2840 isub=1,nrc
      do 2840 iq=1,maxdim
 2840 rawbin(iq,isub)=0.
 2850 continue
      if(iprint.lt.0) go to 2940
c  output all results to this point
      do 2930 ispt=1,2
      if (ispt.eq.1) go to 2860
      if(indvpt.eq.0) go to 2930
      irow1s=1
      irow2s=1
      icol1s=1
      icol2s=indvpt
      go to 2870
 2860 if(irow1.eq.0) go to 2930
      irow1s=irow1+nbord
      irow2s=irow2-nbord
      icol1s=icol1+nbord
      icol2s=icol2-nbord
 2870 do 2920 ira=irow1s,irow2s
      do 2920 ii=icol1s,icol2s
      if(ispt.eq.1) go to 2890
 2880 xouts=xpt1(ii)
      youts=ypt1(ii)
      isub=irows*icols+ii
      go to 2900
c     back transform point from equator to dec degrees
 2890 call backw(ii,ira,xouts,youts)
c     results calculated and printed in subroutine out
      isub=(ira-irow1)*icols+ii-icol1+1
 2900 do 2910 iqr=1,maxp
 2910 reg(iqr)=rawbin(iqr,isub)+smobin(iqr,isub)
      rdis=regg(isub)
c     if(iprint.le.2) write(16,30)tytle
      call out(reg,naf,xouts,youts,sd)
 2920 continue
 2930 continue
 2940 continue
      isw=1
      go to 680
c     finished
 2950 nvals=icols*irows+indvpt
c     write to unit 3 for possible run continuation
c     may delete to save time and space if no run
c     continuations are desired.
      do 2970 i=1,nvals
      do 2960 iqr=1,maxdim
 2960 reg(iqr)=rawbin(iqr,i)+smobin(iqr,i)
      write(3,*) reg
 2970 continue
c    transfer to unit 3 for possible continuation or restart
 2980  continue
 2990 continue
c     write(16,2530) tot5,allerr
 3000 format(/' sources with error in area > .05='f7.0
     & ' out of 'f7.0' sources ')
 3010 format(' number of brute force points='i4)
      stop
      end
